package de.hamster.scratch;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Stack;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import de.hamster.debugger.model.DebuggerModel;
import de.hamster.scratch.elements.voids.FunctionResultException;
import de.hamster.scratch.gui.NextMethodHandler;
import de.hamster.scratch.gui.RefreshHandler;
import de.hamster.workbench.Workbench;

/**
 * Ein ScratchProgramm beinhaltet eine Sequenz von Anweisungen (hier Statements)
 * 
 * @author dibo
 * 
 */
public class ScratchProgram extends Thread {
	private static Renderable active;
	
	private boolean setStopped;
	private boolean setPaused;
	private ScratchHamsterFile file;
	private StorageController program;
	private ArrayList<RefreshHandler> refreshHandler = new ArrayList<RefreshHandler>();
	private ArrayList<NextMethodHandler> nextMethodHandler = new ArrayList<NextMethodHandler>();
	private String openedMethod = "";
	private Stack<String> methodStack = new Stack<String>();

	public ScratchProgram() {
		super();
		this.setStopped = false;
		this.setPaused = false;
		program = new StorageController();
	}
	
	public ScratchProgram(ScratchProgram p) {
		super(p);
		this.setStopped = false;
		this.setPaused = false;
		setProgram(p.getProgram());
	}

	public void setFile(ScratchHamsterFile f) {
		this.file = f;
	}
	
	public void setProgram(StorageController program) {
		this.program = program;
	}
	
	public StorageController getProgram() {
		return program;
	}
	
	public static void setActive(Renderable r) {
		active = r;
	}
	
	public static boolean isActive(Renderable r) {
		return (active == r);
	}
	
	public ArrayList<RefreshHandler> getRefreshHandler() {
		return refreshHandler;
	}
	
	public void setRefreshHandler(ArrayList<RefreshHandler> refreshHandler) {
		this.refreshHandler = refreshHandler;
	}
	
	public void addRefreshHandler(RefreshHandler handler) {
		refreshHandler.add(handler);
	}
	
	public ArrayList<NextMethodHandler> getNextMethodHandler() {
		return nextMethodHandler;
	}
	
	public void setNextMethodHandler(ArrayList<NextMethodHandler> nextMethodHandler) {
		this.nextMethodHandler = nextMethodHandler;
	}
	
	public void addNextMethodHandler(NextMethodHandler handler) {
		nextMethodHandler.add(handler);
	}
	
	public void nexMethod(String name) {
		methodStack.push(name);
	}
	
	public void endMmethod() {
		methodStack.pop();
	}
	
	private void refresh() {
		for (RefreshHandler handler : refreshHandler)
			handler.refresh();
	}
	
	private void onMethodChanged() {
		for (NextMethodHandler handler : nextMethodHandler)
			handler.nextMethod(methodStack.peek());
	}

	public void run() {
		methodStack = new Stack<String>();
		methodStack.push("main");
		openedMethod = "";
		Renderable rootElement = program.getMainRoot();
		try {
			rootElement.perform(this);
		} catch (FunctionResultException e) {
			// Main Funktion wurde mit return verlassen,
			// es soll nichts weiter unternommen werden
		} catch (final Throwable exc) {
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					JOptionPane.showMessageDialog(null, exc.toString(),
							"Scratch-Exception", JOptionPane.ERROR_MESSAGE,
							null);
				}
			});
		} finally {
			active = null;
			refresh();
			this.setStopped = false;
			this.setPaused = false;
			ScratchHamster.getScratchHamster().setProgramFinished();
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					Workbench.getWorkbench().getDebuggerController()
							.getDebuggerModel().setState(
									DebuggerModel.NOT_RUNNING);
					Workbench.getWorkbench().getEditor().getTabbedTextArea()
							.propertyChange(file, false);
				}
			});
		}
	}

	public synchronized void stopProgram() {
		this.notify(); // fr den Fall, dass das Programm gerade pausiert ist
		setStopped = true;
	}

	public synchronized void pauseProgram() {
		setPaused = true;
	}

	public synchronized void resumeProgram() {
		setPaused = false;
		active = null;
		refresh();
		this.notify();
	}

	public synchronized void stepOver() {
		this.notify();
		setPaused = true;
	}

	public synchronized void stepInto() {
		// in dieser Demo gibt es keine Prozeduren
		stepOver(); // Dummy
	}

	public synchronized boolean checkStop() {
		return setStopped;
	}

	public synchronized void checkPause() {
		if (setPaused) {
			if (!openedMethod.equals(methodStack.peek())) {
				openedMethod = methodStack.peek();
				onMethodChanged();
			}
			refresh();
			try {
				wait();
			} catch (InterruptedException exc) {

			}
		}
	}

	public void loadProgram(File xmlFile) {
		this.setStopped = false;
		this.setPaused = false;
		program = new StorageController();
		
		DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
	    domFactory.setNamespaceAware(true);
	    try {
	    	DocumentBuilder builder = domFactory.newDocumentBuilder();
			Document doc = builder.parse(xmlFile);
			NodeList methods = doc.getElementsByTagName("METHOD");
			program.loadProgram(methods);
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		}
	}
}
