package de.hamster.scratch.gui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import javax.swing.JPanel;

import de.hamster.scratch.ScratchPanel;
import de.hamster.scratch.ScratchUtils;

public class TabPanel extends JPanel implements RefreshHandler, TabChangedHandler {
	/**
	 * 
	 */
	private static final long serialVersionUID = 127272748699016187L;
	
	public static Color BACKGROUND_COLOR = new Color(149, 154, 159);
	public static final int CHOSE_BUTTON_WIDTH = 55;
	public static final int START_LEFT = 15;

	private static BufferedImage imgTop;
	private static BufferedImage imgBottom;
	private static BufferedImage imgLeft;
	private static BufferedImage imgRight;
	private static BufferedImage imgTopLeft;
	private static BufferedImage imgTopRight;
	private static BufferedImage imgBottomLeft;
	private static BufferedImage imgBottomRight;
	
	private ArrayList<TabChangedHandler> tabChangedHandler;
	private ArrayList<ContextMenuHandler> contextMenuHandler;
	private ArrayList<MethodChangeHandler> methodChangeHandler;
	private ArrayList<TabOpenedHandler> tabOpenedHandler;
	private ArrayList<TabButton> tabs;
	private ArrayList<TabButton> invisibleButtons;
	private TabButton extraTab;
	private TabButton selected = null;
	
	public TabPanel() {
		loadImages();
		tabs = new ArrayList<TabButton>();
		invisibleButtons = new ArrayList<TabButton>();
		tabChangedHandler = new ArrayList<TabChangedHandler>();
		contextMenuHandler = new ArrayList<ContextMenuHandler>();
		methodChangeHandler = new ArrayList<MethodChangeHandler>();
		tabOpenedHandler = new ArrayList<TabOpenedHandler>();
		
		extraTab = new TabButton("...");
		extraTab.addRefreshHandler(this);
		extraTab.addTabChangedHandler(new ExtraTabClickHandler());
		this.addMouseListener(extraTab);
	}
	
	/**
	 * Wird im Konstruktor aufgerufen und ldt LookAndFeel Bilder.
	 * Diese Methode wird nur einmalig ausgefhrt, da die Bilder
	 * statisch in der Klasse sind.
	 */
	private void loadImages() {
		if (imgTop != null)
			return; // Bilder sind bereits geladen

		imgTop = ScratchUtils.getImage("scratch/backgrounds/top.png");
		imgBottom = ScratchUtils.getImage("scratch/backgrounds/bottom.png");
		imgLeft = ScratchUtils.getImage("scratch/backgrounds/left.png");
		imgRight = ScratchUtils.getImage("scratch/backgrounds/right.png");
		imgTopLeft = ScratchUtils.getImage("scratch/backgrounds/top_left.png");
		imgTopRight = ScratchUtils.getImage("scratch/backgrounds/top_right.png");
		imgBottomLeft = ScratchUtils.getImage("scratch/backgrounds/bottom_left.png");
		imgBottomRight = ScratchUtils.getImage("scratch/backgrounds/bottom_right.png");
	}
	
	@Override
	public void paint(Graphics g) {
		paintBackground(g);
		
		for (TabButton button : tabs)
			button.setVisible(false);
		
		// Zeichne die Buttons
		int left = START_LEFT;
		int selectedLeft = -1;
		for (int i = 0; i < tabs.size(); i++) {
			if (left + tabs.get(i).getWidth() >= getWidth() - CHOSE_BUTTON_WIDTH)
				break;
			
			if (tabs.get(i) == selected) {
				selectedLeft = left;
				left += tabs.get(i).getWidth();
				selected.setVisible(true);
				continue;
			}
			
			tabs.get(i).setVisible(true);
			tabs.get(i).paint(g, left, getHeight() - TabButton.HEIGHT);
			left += tabs.get(i).getWidth();
		}
		
		invisibleButtons.clear();
		for (TabButton button : tabs)
			if (!button.isVisible())
				invisibleButtons.add(button);
		
		if (!invisibleButtons.isEmpty()) {
			extraTab.setVisible(true);
			extraTab.paint(g, getWidth() - 50, getHeight() - TabButton.HEIGHT);
		}
		else
			extraTab.setVisible(false);

		paintBorders(g);
		super.paintComponents(g);
		
		if (selected != null && selectedLeft >= 0)
			selected.paint(g, selectedLeft, getHeight() - TabButton.HEIGHT);
	}
	
	/**
	 * Zeichnet den Hintergrund der Komponente in angegebener
	 * Hintergrundfarbe.
	 * @param g
	 * Das Graphics-Objekt, auf dem gezeichnet werden soll
	 */
	private void paintBackground(Graphics g) {
		g.setColor(BACKGROUND_COLOR);
		g.fillRect(0, 0, getWidth(), getHeight());
	}
	
	/**
	 * Zeichnet die Rnder und Ecken der Komponente mit den dafr
	 * geladenen Bildern. Die Rnder werden zum Schluss gezeichnet,
	 * so dass Transparenzeffekte mglich sind und die darin
	 * enthaltenen Komponenten unter den Rndern erscheinen.
	 * @param g
	 * Das Graphics-Objekt, auf dem gezeichnet werden soll
	 */
	private void paintBorders(Graphics g) {
//		g.drawImage(imgTop, 0, 0, getWidth(), 8, BACKGROUND_COLOR, null);
		for (int i = 0; i < imgTop.getHeight(); i++) {
			g.setColor(new Color(imgTop.getRGB(0, i)));
			g.drawLine(0, i, getWidth(), i);
		}
		
//		g.drawImage(imgBottom, 0, getHeight() - 8, getWidth(), 8, BACKGROUND_COLOR, null);
		for (int i = 0; i < imgBottom.getHeight(); i++) {
			int top = i + getHeight() - imgBottom.getHeight();
			g.setColor(new Color(imgBottom.getRGB(0, i)));
			g.drawLine(0, top, getWidth(), top);
		}
		
//		g.drawImage(imgLeft, 0, 0, 8, getHeight(), BACKGROUND_COLOR, null);
		for (int i = 0; i < imgLeft.getWidth(); i++) {
			g.setColor(new Color(imgLeft.getRGB(i, 0)));
			g.drawLine(i, 0, i, getHeight());
		}
		
//		g.drawImage(imgRight, getWidth() - 5, 0, 5, getHeight(), BACKGROUND_COLOR, null);
		for (int i = 0; i < imgRight.getWidth(); i++) {
			int left = i + getWidth() - imgRight.getWidth();
			g.setColor(new Color(imgRight.getRGB(i, 0)));
			g.drawLine(left, 0, left, getHeight());
		}
		
		g.drawImage(imgTopLeft, 0, 0, null);
		g.drawImage(imgTopRight, getWidth() - imgTopRight.getWidth(), 0, null);
		g.drawImage(imgBottomLeft, 0, getHeight() - imgBottomLeft.getHeight(), null);
		g.drawImage(imgBottomRight, getWidth() - imgBottomRight.getWidth(), getHeight() - imgBottomRight.getHeight(), null);
	}
	
	public void tabChanged(TabButton button) {
		if (selected != null)
			selected.setSelected(false);
		
		selected = button;
		selected.setSelected(true);
		
		if (!selected.isVisible()) {
			tabs.remove(selected);
			tabs.add(0, selected);
			refresh();
		}
		
		for (TabChangedHandler handler : tabChangedHandler)
			handler.tabChanged(button);
	}
	
	public void tabChanged(String name) {
		TabButton button = null;
		for (int i = 0; i < tabs.size(); i++)
			if (tabs.get(i).getName().equals(name)) {
				button = tabs.get(i);
				break;
			}
		
		tabChanged(button);
	}
	
	/**
	 * Fgt einen TabChangedHandler zu diesem TabPanel hinzu, dessen
	 * interne Methode tabChanged aufgerufen wird, sobald sich der
	 * aktive Tab ndert.
	 * @param handler
	 * Handler, der beim Tabwechsel aufgerufen werden soll
	 */
	public void addTabChangedHandler(TabChangedHandler handler) {
		tabChangedHandler.add(handler);
	}
	
	/**
	 * Fgt einen TabOpenedHandler zu diesem TabPanel hinzu, dessen
	 * interne Methode tabChanged aufgerufen wird, sobald ein Tab
	 * geschlossen wird
	 * @param handler
	 * Handler, der beim Tabwechsel aufgerufen werden soll
	 */
	public void addTabOpenedHandler(TabOpenedHandler handler) {
		tabOpenedHandler.add(handler);
	}
	
	/**
	 * Fgt einen MethodChangeHandler zu diesem TabPanel hinzu, dessen
	 * interne Methode onDelete oder onRename aufgerufen werden, sobald
	 * sich diese Menupunkte aus dem Contextmenu aufgerufen wurden.
	 * @param handler
	 * Handler, der beim Lschen oder Umbenennen aufgerufen werden soll
	 */
	public void addMethodChangeHandler(MethodChangeHandler handler) {
		methodChangeHandler.add(handler);
	}
	
	/**
	 * Fgt einen ContextMenuHandler zu diesem TabPanel hinzu, dessen
	 * interne Methode openContextMenu aufgerufen wird, sobald ein
	 * Menu aufgerufen wird.
	 * @param handler
	 * Handler
	 */
	public void addContextMenuHandler(ContextMenuHandler handler) {
		contextMenuHandler.add(handler);
	}
	
	private void openContextMenu(TabButton sender, int x, int y) {
		ArrayList<MenuItem> items = new ArrayList<MenuItem>();
		
		MenuItem temp = new MenuItem("Schlieen", new CloseTabClickHandler(sender));
		temp.setEnabled(tabs.size() != 1);
		items.add(temp);
		
		MenuItem temp2 = new MenuItem("Andere schlieen", new CloseAllTabsExpectClickHandler(sender));
		temp2.setEnabled(tabs.size() != 1);
		items.add(temp2);
		
		items.add(new MenuItemSeparator());
		
		MenuItem renameItem = new MenuItem("Umbenennen", new RenameMethodClickHandler(sender.getName()));
		renameItem.setEnabled(!sender.getName().equals("main"));
		items.add(renameItem);
		
		MenuItem deleteItem = new MenuItem("Lschen", new DeleteMethodClickHandler(sender));
		deleteItem.setEnabled(!sender.getName().equals("main"));
		items.add(deleteItem);
		
		for (ContextMenuHandler handler : contextMenuHandler)
			handler.openContextMenu(items, x + ScratchPanel.OPTIONS_WIDTH - MenuItem.PADDING, y + MenuItem.PADDING);
 	}
	
	private void openNotVisibleContextMenu() {
		ArrayList<MenuItem> items = new ArrayList<MenuItem>();
		
		for (TabButton button : tabs)
			if (!button.isVisible())
				items.add(new MenuItem(button.getName(), new ExtraMenuItemClickHandler(button)));
		
		for (ContextMenuHandler handler : contextMenuHandler)
			handler.openContextMenu(items, getWidth() + ScratchPanel.OPTIONS_WIDTH - 20, ScratchPanel.TAB_PANEL_HEIGHT + 5);
	}
	
	/**
	 * Fgt einen neuen TabButton zu diesem TabPanel hinzu
	 * @param name
	 * Text auf dem Button
	 */
	public void addTab(String name) {
		// berprfe, ob dieser Tab bereits existiert
		if (existsTab(name)) {
			tabChanged(name);
			return;
		}
		
		onTabOpened(name, true);
		TabButton button = new TabButton(name);
		tabs.add(button);
		this.addMouseListener(button);
		button.addRefreshHandler(this);
		button.addTabChangedHandler(this);
		button.addTabClickHandler(new TabButtonRightClickHandler());
		tabChanged(button);
		
		repaint();
	}
	
	public boolean existsTab(String name) {
		for (TabButton tab : tabs)
			if (tab.getName().equals(name))
				return true;
		
		return false;
	}
	
	public TabButton getTab(String name) {
		for (TabButton tab : tabs)
			if (tab.getName().equals(name))
				return tab;
		
		return null;
	}
	
	/**
	 * Schliesst den bergebenen Tab.
	 * Vorausgesetzt es ist nicht der letzte.
	 * @param button
	 */
	public void closeTab(TabButton button) {
		this.removeMouseListener(button);
		tabs.remove(button);
		onTabOpened(button.getName(), false);
		
		if (tabs.size() <= 0) {
			addTab("main");
			onTabOpened("main", true);
		}
		
		if (selected == button)
			tabChanged(tabs.get(0));
		
		refresh();
	}
	
	/**
	 * Schliesst den bergebenen Tab.
	 * Vorausgesetzt es ist nicht der letzte, ansonsten wird main geffnet.
	 * @param button
	 */
	public void closeTab(String name) {
		TabButton button = getTab(name);
		if (button == null)
			return;
		
		closeTab(button);
	}
	
	public void renameTab(String fromName, String toName) {
		for (TabButton tab : tabs)
			if (tab.getName().equals(fromName))
				tab.setName(toName);
		
		refresh();
	}
	
	/**
	 * Schliesst alle Tabs bis auf den bergebenen Tab.
	 * @param button
	 */
	public void closeAllTabsExcept(TabButton button) {
		while (!tabs.isEmpty()) {
			TabButton temp = tabs.remove(0);
			this.removeMouseListener(temp);
			onTabOpened(temp.getName(), false);
		}
		
		tabs.add(button);
		this.addMouseListener(button);
		tabChanged(button);
		onTabOpened(button.getName(), true);
		
		refresh();
	}
	
	@Override
	public void refresh() {
		repaint();
	}
	
	public void onTabOpened(String name, boolean opened) {
		for (TabOpenedHandler handler : tabOpenedHandler)
			handler.tabOpened(name, opened);
	}
	
	class ExtraTabClickHandler implements TabChangedHandler {
		@Override
		public void tabChanged(TabButton button) {
			openNotVisibleContextMenu();
		}
	}
	
	class TabButtonRightClickHandler implements TabClickHandler {
		@Override
		public void onClick(TabButton sender, int x, int y) {
			openContextMenu(sender, x, y);
		}
	}
	
	class ExtraMenuItemClickHandler implements ClickHandler {
		private TabButton button;
		
		public ExtraMenuItemClickHandler(TabButton button) {
			this.button = button;
		}
		
		@Override
		public void onClick() {
			tabChanged(button);
		}
	}
	
	class CloseTabClickHandler implements ClickHandler {
		private TabButton button;
		
		public CloseTabClickHandler(TabButton button) {
			this.button = button;
		}
		
		@Override
		public void onClick() {
			closeTab(button);
		}
	}
	
	class CloseAllTabsExpectClickHandler implements ClickHandler {
		private TabButton button;
		
		public CloseAllTabsExpectClickHandler(TabButton button) {
			this.button = button;
		}
		
		@Override
		public void onClick() {
			closeAllTabsExcept(button);
		}
	}
	
	class DeleteMethodClickHandler implements ClickHandler {
		private TabButton button;
		
		public DeleteMethodClickHandler(TabButton button) {
			this.button = button;
		}

		@Override
		public void onClick() {
			for (MethodChangeHandler handler : methodChangeHandler)
				handler.onDelete(button);
		}
	}
	
	class RenameMethodClickHandler implements ClickHandler {
		private String name;
		
		public RenameMethodClickHandler(String name) {
			this.name = name;
		}

		@Override
		public void onClick() {
			for (MethodChangeHandler handler : methodChangeHandler)
				handler.onRename(name);
		}
	}
}
